#!/bin/sh

# Illustration of how to use the BMP085 pressure sensor with the sensor20 program.
# Note that it is a 3.3v IC and can't be powered directly from the 5v bus.
# The pullup resistors must also be attached to 3.3v, not 5v.

export SCL=AN6
export SDA=AN5

OSS=0
MAX=10

[ -z "$1" ] || OSS=$1
[ -z "$2" ] || MAX=$2

CMD=`printf "%02x" $((0x34 + (OSS<<6) ))`
CALIBRATION=$(./sensor20 77[aa]:22)

# Read four hex-digits a time into the listed variables, converting from hex to decimal
N=-4
for X in AC1 AC2 AC3 AC4 AC5 AC6 B1 B2 MB MC MD
do
	read $X <<EOF
$((0x${CALIBRATION:$((N+=4)):4}))
EOF
done

# The following variables are signed 16-bit.  The others are already unsigned.
for X in AC1 AC2 AC3 B1 B2 MB MC MD
do
	V=${!X}
	[ $V -gt 32767 ] && ((V=(V&0x7fff)-32768))
	read $X <<EOF
$V
EOF
done

for((X=0; X<MAX; X++))
do
UT=$((0x$(./sensor20 77[f4]=2e D:5 77[f6]:2)))
UP=$((0x$(./sensor20 77[f4]=$CMD D:25 77[F6]:3) >> (8-OSS)))

X1=$(( ((UT-AC6)*AC5)/32768 ))
X2=$(( (MC*2048)/(X1+MD) ))
B5=$(( X1+X2 ))
T=$(((B5+8)/16))

B6=$((B5-4000))
X1=$(( (B2*(B6*B6/(1<<12) )) / (1<<11) ))
X2=$(( AC2*B6/(1<<11) ))
X3=$((X1+X2))
B3=$(( (((AC1*4)+X3)<<OSS)/4 ))
X1=$(( AC3*B6/(1<<13) ))
X2=$(( (B1*(B6*B6/(1<<12)))/(1<<16) ))
X3=$(( ((X1+X2)+2)/4 ))
B4=$(( AC4*(X3+32768)/(1<<15) ))
B7=$(( (UP-B3)*(50000>>OSS) ))
if [ "$B7" -gt $((0x80000000)) ]
then
	p=$(( (B7*2)/B4 ))
else
	p=$(( (B7/B4)*2 ))
fi

X1=$(( (p/(1<<8))*(p/(1<<8)) ))
X1=$(( (X1*3038) / (1<<16) ))
X2=$(( (-7357*p)/(1<<16) ))
p=$(( p + (X1+X2+3791)/(1<<4) ))

# echo $CALIBRATION
# echo $AC1 $AC2 $AC3 $AC4 $AC5 $AC6 $B1 $B2 $MB $MC $MD

printf "Temperature:	%d.%02dC\tPressure:\t%d.%3d kPa\n"		\
	$((T/10)) $((T%10))						\
	$((p/1000)) $(( p%1000 ))
done
