#!/bin/bash

# Example of talking to a co2meter.com K30 CO2 sensor over I2C
# Note that these come with internal 3.3v pullups, so the LED trick
# isn't even needed.

export SDA=AN11
export SCL=AN10

die () {
	echo "$@" >&2
	exit 1
}

add_hex () {
	# Add to string direct
	DATA="$DATA$1"

	# Calculate checksum by adding each byte
	while [ "${#1}" -gt 0 ]
	do
		((CKSUM	+= 0x${1:0:2}))
		((CKSUM &= 0xff))
		set -- ${1:2}
	done
}

DADDR="68"	# I2C address.  They also respond to 7f.
CMMD="2"	# Command 0x2, "read RAM"
BYTES="2"	# Read 0x2 bytes of RAM
RADDR="0008"	# Read from address 0x0008, i.e. CO2 value register

CKSUM=0		# Checksum starts at 0
DATA=""		# Data starts blank

# Add all bytes to the string in order
add_hex $CMMD$BYTES
add_hex $RADDR
# Add our own checksum to the string
add_hex $(printf %02x $CKSUM)

CMD="$DADDR[]=$DATA $DADDR[]:$((2+0x$BYTES))"

echo DATA is $DATA

REPLY=$(sensor20 $DADDR[]=$DATA $DADDR[]:$((2+0x$BYTES)))

[ ${REPLY:0:1} = $CMMD	] || die "Reply($REPLY) not type $CMMD"
[ ${REPLY:1:1} = 1	] || die "Reply($REPLY) not complete"

CKSUM=0
add_hex ${REPLY:0:$((${#REPLY}-2))}	# Calculate checksum
# echo "Reply $REPLY checksum $(printf %02x $CKSUM)"

VAL=$((0x${REPLY:2:4}))
# Signed 16-bit, so if bit 16 set, subtract 32768+32768
[ "$VAL" -gt 32768 ] && ((VAL -= 65536))

echo "$VAL PPM CO2 detected"
