#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "i2c.h"
#include "mydebug.h"

static int started = 1, i2cerr=0;

// Minimum delay is 4.7 microseconds, MAXIMUM speed of sensor20 is 50 microseconds.
// No delay needed, so this becomes a do-nothing stub.
#define I2C_delay() { }

static void pin_clear(int FD, pin_t pin)	{	cmd20_out(FD, pin, 0);	}
static int pin_read(int FD, pin_t pin) { int bit=0; cmd20_in(FD, pin, &bit); return(bit); }
int i2c_start_cond(int FD, int SCL, int SDA);
int i2c_stop_cond(int FD, int SCL, int SDA);
int cmd20_i2c_restart(void) { started=0; }

static int CLOCKSTRETCH(int FD, int SCL)
{
	int n;
	for(n=0; n<100; n++)
	if(pin_read(FD, SCL) == 1) return(0);

	return(-1);	// Timeout
}

//#define CLOCKSTRETCH()  while (pin_read(FD, SCL) == 0);  // Clock stretching

#define ARBITRATION_QUIT(X) { errmsg=(X); goto I2C_ARBITRATION_FAIL; }

static void arbitration_lost(int fd, pin_t SDA, pin_t SCL)
{
	const char *errmsg=NULL;
	int tries=16;
	static int recur=0;

	recur++;

	if(recur>1)
		return;

	i2cerr++;

	INFO_PRINT("Arbitration Lost, SDA:%d SCL:%d\n",
		pin_read(fd, SDA), pin_read(fd, SCL));

	if(pin_read(fd, SCL) == 0)
		ARBITRATION_QUIT("Slave holds clock low");

	while((pin_read(fd, SDA) == 0)&&(tries-- > 0))
	{
		pin_clear(fd, SCL);
		pin_read(fd, SCL);
	}

	if((tries <= 0) && (pin_read(fd, SDA) == 0))
		ARBITRATION_QUIT("Slave still hogging SDA");

	if(i2c_start_cond(fd, SCL, SDA) < 0)
		ARBITRATION_QUIT("I2C start lost arbitration");

	if(i2c_stop_cond(fd, SCL, SDA) < 0)
		ARBITRATION_QUIT("I2C stop lost arbitration");

I2C_ARBITRATION_FAIL:
	if(errmsg != NULL)
	INFO_PRINT("Couldn't retake arbitration SCL[%d] SDA[%d] %s\n",
		SCL, SDA, errmsg);

	recur=0;
	return;	
}

#define NACK 1
#define START 1
#define STOP 1

static unsigned char i2c_read_byte(int fd, pin_t SCL, pin_t SDA, int nack, int send_stop);
static int i2c_write_byte(int fd, pin_t SCL, pin_t SDA, int send_start, int send_stop, unsigned char byte);

// 7-bit i2c writes without register address.
int cmd20_i2c_write7raw(int fd, pin_t scl, pin_t sda, int devid,
        unsigned char *bytes, int count)
{
	int pos=0;

	if(i2c_write_byte(fd, scl, sda, START, !STOP, (devid<<1)|0) != 0)
	{
		DEBUG_PRINT(("first byte fail\n"));
		return(-1);
	}

	DEBUG_PRINT(("Got NACK for addr...\n"));

	while((count--) > 1)
	if(i2c_write_byte(fd,scl,sda,!START,!STOP,bytes[pos++])!=0)
	{
		DEBUG_PRINT((" data write failed\n"));
		return(-1);
	}

	if(i2c_write_byte(fd,scl,sda,!START,STOP,bytes[pos++])!=0)
	{
		DEBUG_PRINT(("last write failed\n"));
		return(-1);
	}
	started=0;

	return(0);
}


// 7-bit I2C reads without register address.
int cmd20_i2c_read7raw(int fd, pin_t scl, pin_t sda, int devid, int count)
{
	if(i2c_write_byte(fd,scl,sda,START,!STOP,(devid<<1)|1) != 0)
		return(-1);

	while(count > 1)
	{
		printf("%02x", i2c_read_byte(fd, scl, sda, !NACK, !STOP));
		count--;
	}

	printf("%02x\n", i2c_read_byte(fd,scl,sda,NACK,STOP));
	return(0);
}

int cmd20_i2c_write7_multi(int fd, pin_t scl, pin_t sda, int devid, int reg, 
	const unsigned char *buf, int len)
{
	i2cerr=0;

	if(i2c_write_byte(fd,scl,sda,START,!STOP,(devid<<1)|0) != 0)
		return(-1);
	if(i2c_write_byte(fd,scl,sda,!START, !STOP, reg) != 0)
		return(-1);

	while(len > 1)
	{
		if(i2c_write_byte(fd, scl, sda, !START, !STOP,  *(buf++)) != 0)
			return(-1);

		if(i2cerr) return(-1);

		len--;
	}

	if(i2c_write_byte(fd,scl,sda,!START, STOP, *buf) != 0)
		return(-1);

	if(i2cerr) return(-1);

	return(0);
}

int cmd20_i2c_read7_multi(int fd, pin_t scl, pin_t sda, int devid, int reg, int len)
{
	unsigned char byte;
	i2cerr=0;

	if(i2c_write_byte(fd,scl,sda,START,!STOP,(devid<<1)|0) != 0)
		return(-1);
	if(i2c_write_byte(fd,scl,sda,!START, !STOP, reg) != 0)
		return(-1);
	if(i2c_write_byte(fd,scl,sda,START, !STOP, (devid<<1)|1) != 0)
		return(-1);

	while(len > 1)
	{
		byte=i2c_read_byte(fd, scl, sda, !NACK, !STOP);
		if(i2cerr) return(-1);

		printf("%02x", byte);
		len--;
	}

	byte=i2c_read_byte(fd, scl, sda, NACK, STOP);
	if(i2cerr) return(-1);
	printf("%02x\n", byte);
	return(0);
}

int cmd20_i2c_write7(int fd, pin_t scl, pin_t sda, int devid, int reg, int byte)
{
	i2cerr=0;

	if(i2c_write_byte(fd,scl,sda,START,!STOP,(devid<<1)|0) != 0)
		return(-1);
	if(i2c_write_byte(fd,scl,sda,!START, !STOP, reg) != 0)
		return(-1);
	if(i2c_write_byte(fd,scl,sda,!START, STOP, byte) != 0)
		return(-1);

	if(i2cerr) return(-1);

	return(0);
}

int cmd20_i2c_read7(int fd, pin_t scl, pin_t sda, int devid, int reg)
{
	unsigned char byte;

	i2cerr=0;

	if(i2c_write_byte(fd,scl,sda,START,!STOP,(devid<<1)|0) != 0)
		return(-1);
	if(i2c_write_byte(fd,scl,sda,!START, !STOP, reg) != 0)
		return(-1);
	if(i2c_write_byte(fd,scl,sda,START, !STOP, (devid<<1)|1) != 0)
		return(-1);
	byte=i2c_read_byte(fd,scl,sda,NACK,STOP);
	if(i2cerr) return(-1);

	printf("%02x\n", byte);
	return(0);
}

int cmd20_i2c(int fd, pin_t scl, pin_t sda, char *cmd)
{
	int x;
	const char *arg=NULL;
	unsigned char addr=0;

	i2cerr=0;

	arg=strtok(cmd, ",");

	while(arg)
	{
		const char *a=arg;
		char g;
		int addr=0;
		int start=0, stop=0, read=0, nack=0;
		int ret=0;
		int data;

		while(*a)
		{
			if((*a) == 'S')		start=1;
			else if((*a) == 'P')	stop=1;
			else if((*a) == 'A')	addr=1;
			else if((*a) == 'R')	read=1;
			else if((*a) == 'W')	read=0;
			else if((*a) == 'N')	nack=1;
			else if((*a) == 't')	started=0;
			else if((*a) == 'T')	started=1;
			else			break;
			a++;
		}

		if(addr)
		{
			sscanf(a, "%02x", &data);
			ret=i2c_write_byte(fd, scl, sda, start, stop, (data << 1) | read);
		}
		else if(read)
		{
			data=i2c_read_byte(fd, scl, sda, nack, stop);
			printf("%02x\n", data);
			ret=0;
		}
		else
		{
			sscanf(a, "%02x", &data);
			ret=i2c_write_byte(fd, scl, sda, start, stop, data);
		}

		if(ret != 0)
			fprintf(stderr, "'%s' failed\n", arg);

#if 0
		// 7-bit address r/w, i.e. A7R1c, A7W1c
		if(sscanf(arg, "A7R%02x%c", &addr, &g) == 1)
			i2c_write_byte(1, 0, (addr<<1) | 1);
		else if(sscanf(arg, "A7W%02x%c", &addr, &g) == 1)
			i2c_write_byte(1, 0, (addr<<1));
		else if(strcmp(arg, "r") == 0)
			printf("%02x\n", i2c_read_byte(0, 0));
		else if(strcmp(arg, "R") == 0)
			printf("%02x\n", i2c_read_byte(1, 1));
		else if(sscanf(arg, "w%02x%c", &addr, &g) == 1)
			i2c_write_byte(0, 0, addr);
		else if(sscanf(arg, "W%02x%c", &addr, &g) == 1)
			i2c_write_byte(1, 1, addr);
		else	fprintf(stderr, "Uknown i2c command '%s'\n", arg);
#endif

		arg=strtok(NULL, ",");
	}

	if(i2cerr) return(-1);

	return(0);

#if 0
	fprintf(stderr, "Printing register map!\n");

	i2c_write_byte(1, 0, WRITE_7BIT(addr));
	i2c_write_byte(0, 0, 0);
	i2c_write_byte(1, 0, READ_7BIT(addr));
	for(x=0; x<0x39; x++)
	{
		printf("%02x] %d\n", x, i2c_read_byte(0, 0));
	}

	printf("%02x] %d\n", x, i2c_read_byte(1, 1));
	return(0);
#endif
}
 
int i2c_start_cond(int FD, int SCL, int SDA) {

  if (started) { // if started, do a restart cond
    // set SDA to 1
    pin_read(FD, SDA);
    I2C_delay();
    if(CLOCKSTRETCH(FD, SCL) < 0)
      return(-1);

    // Repeated start setup time, minimum 4.7us
    I2C_delay();
  }

  if (pin_read(FD, SDA) == 0) {
    arbitration_lost(FD, SDA, SCL);
    return(-1);
  }
  // SCL is high, set SDA from 1 to 0.
  pin_clear(FD, SDA);
  I2C_delay();
  pin_clear(FD, SCL);
  started = 1;
  return(0);
}
 
int i2c_stop_cond(int FD, int SCL, int SDA){
  // set SDA to 0
  pin_clear(FD, SDA);
  I2C_delay();
  // Clock stretching
  if(CLOCKSTRETCH(FD, SCL) < 0)
    return(-1);

  // Stop bit setup time, minimum 4us
  I2C_delay();
  // SCL is high, set SDA from 0 to 1
  if (pin_read(FD, SDA) == 0) {
    arbitration_lost(FD, SDA, SCL);
    return(-1);
  }
  I2C_delay();
  started = 0;
  return(0);
}
 
// Write a bit to I2C bus
void i2c_write_bit(int FD, int SCL, int SDA, int bit) {
  if (bit) {
    pin_read(FD, SDA);
  } else {
    pin_clear(FD, SDA);
  }
  I2C_delay();
  CLOCKSTRETCH(FD, SCL);
  // SCL is high, now data is valid
  // If SDA is high, check that nobody else is driving SDA
  if (bit && pin_read(FD, SDA) == 0) {
    arbitration_lost(FD, SDA, SCL);
  }
  I2C_delay();
  pin_clear(FD, SCL);
}
 
// Read a bit from I2C bus
int i2c_read_bit(int FD, int SCL, int SDA) {
  int bit;
  // Let the slave drive data
  pin_read(FD, SDA);
  I2C_delay();
  CLOCKSTRETCH(FD, SCL);
  // SCL is high, now data is valid
  bit = pin_read(FD, SDA);
  I2C_delay();
  pin_clear(FD, SCL);
  return bit;
}
 
// Write a byte to I2C bus. Return 0 if ack by the slave.
int i2c_write_byte(int FD, int SCL, int SDA, int send_start,
                    int send_stop,
                    unsigned char byte) {

  DEBUG_PRINT(("%s(%d, %d, 0x%02x)\n", __FUNCTION__, send_start, send_stop, byte));

  unsigned bit;
  int nack;
  if (send_start) {
    i2c_start_cond(FD, SCL, SDA);
  }
  for (bit = 0; bit < 8; bit++) {
    i2c_write_bit(FD, SCL, SDA, (byte & 0x80) != 0);
    byte <<= 1;
  }
  nack = i2c_read_bit(FD, SCL, SDA);
  if (send_stop) {
    i2c_stop_cond(FD, SCL, SDA);
  }
  return nack;
}
 
// Read a byte from I2C bus
unsigned char i2c_read_byte(int FD, pin_t SCL, pin_t SDA, int nack, int send_stop) {
  DEBUG_PRINT(("%s(%d, %d)\n", __FUNCTION__, nack, send_stop));
  unsigned char byte = 0;
  unsigned bit;
  for (bit = 0; bit < 8; bit++) {
    byte = (byte << 1) | i2c_read_bit(FD, SCL, SDA);
  }
  i2c_write_bit(FD, SCL, SDA, nack);
  if (send_stop) {
    i2c_stop_cond(FD, SCL, SDA);
  }
  return byte;
}
