/**
 *	Header information for DLP-IO20 USP data acquisition modules.
 *	Written by Tyler Montbriand in 2012.  Freeware.
 */
#ifndef __SENSOR20_H__
#define __SENSOR20_H__

enum {
	CMD_PING	=	0x27,
	CMD_FLASHLED	=	0x28,
	CMD_LEDCTL	=	0x29,
	CMD_RELAYCTL	=	0x30,
	CMD_IOCMD	=	0x35,
	CMD_ENABLECTR	=	0x36,
	CMD_READCTR	=	0x37,
	CMD_DETECTSENSOR=	0x39,
	CMD_CONVERTSENSOR=	0x40,
	CMD_READSENSOR	=	0x41,
	CMD_SETRES	=	0x42,
	CMD_ANALOGREAD	=	0x50,
	CMD_MULTIREAD	=	0x51,
	CMD_STREAM	=	0x52,
	CMD_EXTERNALREF	=	0x53,
	CMD_INTERNALREF	=	0x54,	
};

enum {
	B2_RELAY_1	=	1,
	B2_RELAY_2	=	2,
	B3_RELAY_SET	=	0,
	B3_RELAY_RESET	=	1,

	B3_OUTPUT	=	0,
	B3_INPUT	=	1,

//	B2_CHANNEL_RB6	=	6,
//	B2_CHANNEL_RB7	=	7,
	B3_FALLING_EDGE	=	0,
	B3_RISING_EDGE	=	1,
};

enum {
	// Might get this on a bad temp read
	TEMP_ERROR	=0xffff,
	// Might get this on the first read
	TEMP_RESET	=0x0550,
	// 125 degrees C / .0625 degrees per bit
	TEMP_MAX	=2000,
	// -55 degrees C / .0625 degrees per bit
	TEMP_MIN	=-880
};

typedef enum pin_t {
	PIN_FAIL=-1,
	AN0, AN1, AN2, AN3,
	AN4, AN5, AN6, AN7,
	AN8, AN9,AN10,AN11,
	AN12,AN13,
	RA4,
	P5, P6, P7,
	RB7, RB6,
	PIN_MAX,
} pin_t;

typedef enum samples_t {
	SAMPLES_128,	SAMPLES_256,
	SAMPLES_512,	SAMPLES_1024,
	SAMPLES_2048,	SAMPLES_4096,
	SAMPLES_8192,
	SAMPLES_END,
} samples_t;

typedef enum rate_t {
	RATE_1K,	RATE_2K,
	RATE_4K,	RATE_8K,
	RATE_20K,
	RATE_END,
} rate_t;

extern const char const *pintab[RB6+2];
extern const int sample_size[SAMPLES_END+1];
extern const char const *sample_rate[RATE_END+1];
extern const char const *sample_size_c[SAMPLES_END+1];
extern int cmd20_err;

typedef enum cmd20_err_t {
	CMD20_ERR_NONE	=	0,
	CMD20_ERR_READ,
	CMD20_ERR_WRITE,
	CMD20_ERR_PARAM,
	CMD20_ERR_NOTREADY,
	CMD20_ERR_SHORT,
	CMD20_ERR_NOSENSOR,
	CMD20_ERR_NODETECT,
} cmd20_err_t;

extern const char const *cmd20_errors[CMD20_ERR_NODETECT+1];


int cmd20_ping(int fd);
int cmd20_flash(int fd);
int cmd20_ledctl(int fd, int led);
int cmd20_relay(int fd, int which, int where);
int cmd20_out(int fd, pin_t pin, int value);
int cmd20_in(int fd, pin_t pin, int *value);
#define CMD20_TRISTATE(FD, PIN)	cmd20_in(FD, PIN, NULL)
int cmd20_opendrain(int fd, pin_t pin, int value, int *result);

int cmd20_enablectr(int fd, pin_t pin, int edge);
int cmd20_readctr(int fd, pin_t pin, unsigned int *value);

int cmd20_18b20_detect(int fd, pin_t pin, unsigned char serial[8]);
int cmd20_18b20_start(int fd, pin_t pin);
int cmd20_18b20_read(int fd, pin_t pin, short *temp);

int cmd20_voltage(int fd, pin_t pin, unsigned short *result);
int cmd20_multivoltage(int fd, pin_t pin, rate_t rate,
	samples_t samples, unsigned short *result);

int cmd20_oversample(int fd, pin_t pin, rate_t rate, samples_t samples,
	unsigned int *total);

int cmd20_externalref(int fd);
int cmd20_internalref(int fd);

int cmd20_compile(unsigned char *buf, int *pos, int max, ...);

#endif/*__SENSOR20_H__*/
