/**
 * tsm_getopt.c:  Automatic commandline option parser.
 * Copyright 2009 Tyler Montbriand, under the GNU LGPL.
 */

#include <stdio.h>
#include "tsm_getopt.h"

static int tsm_findarg(const char *str, const opt options[])
{
	int n;
	if(str[1] == '\0')
		return(-1);

	for(n=0; options[n].lname != NULL; n++)
	{
		if((options[n].sname) &&
			(str[1] == options[n].sname) &&
			(str[2] == '\0'))
			return(n);

		if(str[1] == '-')
		if(strcmp(str+2, options[n].lname)==0)
			return(n);
	}

	return(-1);
}

int tsm_usage(int argc, char *argv[], const opt options[])
{
	int n;

	fprintf(stderr, "Usage:  %s ...\n", argv[0]);

	for(n=0; options[n].lname != NULL; n++)
	{
		int m;
		fprintf(stderr, "\t--%s", options[n].lname);
		if(options[n].sname)
			fprintf(stderr,"(-%c)", options[n].sname);

		if(options[n].args)	fprintf(stderr, "\tVAL");
		else			fprintf(stderr, "\t");

		if(options[n].usage)
			fprintf(stderr, "\t%s", options[n].usage);
		fprintf(stderr, "\n");
		if(options[n].strlist)
		{
			fprintf(stderr,"\t\tValid values");
			for(m=0; options[n].strlist[m]; m++)
				fprintf(stderr, " %s", options[n].strlist[m]);
			fprintf(stderr, "\n");
		}
	}
}

int tsm_getopt(int argc, char *argv[], int arg, const opt options[])
{
	static int stopped=0;
//	int arg=1;

	if(stopped)	return(arg);

	while(arg < argc)
	{
		int which;
		if(strcmp(argv[arg], "--") == 0)
		{
			stopped=1;
			return(arg+2);
		}
		else if(argv[arg][0] == '-')
		{
			which=tsm_findarg(argv[arg], options);
			if(which < 0)
			{
				fprintf(stderr, "invalid option '%s'\n", 
					argv[arg]);
				return(-1);
			}

			else if(options[which].args)
			{
				if((argc - arg) < 2)
				{
					fprintf(stderr, "Need argument for %s\n",
						argv[arg]);
					return(-1);
				}

				// Store if possible
				if(options[which].val)
				{
					if(options[which].strlist)
					{
						int x;
						for(x=0; options[which].strlist[x]; x++)
						if(strcmp(options[which].strlist[x], argv[arg+1]) == 0)
							break;

						if(options[which].strlist[x] == NULL)
						{
							fprintf(stderr, "'%s' Not in enum list\n", argv[arg+1]);
							fprintf(stderr, "valid options are");
							for(x=0; options[which].strlist[x]; x++)
								fprintf(stderr, " %s", options[which].strlist[x]);
							fprintf(stderr, "\n");
							return(-1);
						}

						(*(int *)options[which].val)=x;
					}
					else if(options[which].cmdstr)
					{
						if(sscanf(argv[arg+1], 
							options[which].cmdstr, 
							options[which].val) != 1)
						{
							fprintf(stderr, "Invalid value "
								"'%s' for option '%s'\n",
								argv[arg], argv[arg+1]);
							return(-1);
						}
					}
					else
						*((char **)options[which].val)=argv[arg+1];

					arg++;
				}
			}
			else if(options[which].val != NULL)
			{
				// flag if possible
				*((int *)options[which].val)=1;
			}

			arg++;
		}
		else
		{
			return(arg);
		}
	}

	return(arg);
}
